import { Observable, Subject } from "rxjs";
import { RetransmissionService } from "../utils/retransmission.service";

export type TransportManagerEvent = 'New Client Connection' | 'New Client Request' | 'Client Disconnected'
export type TransportEvent = 'Connection' | 'New Message' | 'Disconnection'
export type TransportType = 'WEBSOCKET' | 'GRPC' | 'HTTP' | 'KAFKA' | undefined

export interface TransportManager {
    getTransportManagerEventNotification(): Observable<TransportManagerEventNotification>
}
export interface ITransport {
    getTransportEventNotification(): Observable<TransportEventNotification>
}

export interface ITransportTransmitting extends ITransport {
    transmit(message: TransportMessage): void
}

export interface ITransportReceiving extends ITransport {
    receive(): Observable<TransportMessage>
}

export interface ITransportSuper extends ITransportTransmitting, ITransportReceiving {
    // additional properties && methods  here
}
export interface TransportSettings {
    profileInfo: ProfileInfo,
}

export interface ProfileInfo {
    id: string,
    name: string,
    transport: TransportType,
    url?: string | null,
    port?: number | null
}

export interface TransmitterProfile {
    uuid: string,
    name: string,
    dateCreated: Date,
    transportType: TransportType,
    eventNotification: Subject<TransportEventNotification>,
    instance?: any
}

export interface ReceiverProfile {
    uuid: string,
    name: string,
    dateCreated: Date,
    transportType: TransportType,
    eventNotification: Subject<TransportEventNotification>,
    instance?: any
}

export interface ReceiverRetransmissionProfile extends ReceiverProfile {
    buffer: RetransmissionService
}

export interface TransportEventNotification {
    event: TransportEvent,
    description: string,
    transportType: TransportType,
    data?: TransportPayload
}

export interface TransportManagerEventNotification {
    event: TransportManagerEvent,
    data?: TransportPayload
}
export interface TransportMessage {
    id: string,
    receiverID: string,
    payload: any,
    event: string
}

export interface TransportPayload {
    receiverID: string,
    receiverName: string,
    date: Date,
    payload?: any
}


export interface WrappedMessage {
    timeReceived: any, // this property is for sender to sort
    payload: any,
    thisMessageID?: string,
    previousMessageID?: string | null // this property is for receiver to sort
}